#include "windowproperties.h"

#include <QByteArray>
#include <QX11Info>
#include <X11/Xlib.h>
#include <X11/Xutil.h>

WindowProperties::WindowProperties(Window w) :
    w(w)
{

}

QMap<QString,QVariant> WindowProperties::CurrentProperties()
{
    QMap<QString,QVariant> ret;
    int numProps = 0;
    Atom* props = XListProperties(QX11Info::display(), w, &numProps);
    for (int i = 0; i < numProps; ++i) {
        QString propKey = LookupAtom(props[i]);

        // Look up property value
        // Request 0 bytes of it, to determine total property length
        Atom actualType;
        int actualFormat;
        unsigned long bytesRead;
        unsigned long bytesRemaining;
        unsigned char* data;
        XGetWindowProperty(QX11Info::display(), w, props[i], 0L, 0L, false, AnyPropertyType, &actualType, &actualFormat, &bytesRead, &bytesRemaining, &data);
        // FIXME assert that it returns Success

        if (data != nullptr) {
            XFree(data);
        }

        // Now that we know the full data length, request it properly
        XGetWindowProperty(QX11Info::display(), w, props[i], 0L, bytesRemaining, false, AnyPropertyType, &actualType, &actualFormat, &bytesRead, &bytesRemaining, &data);

        QByteArray qba((char*)data, (long)bytesRead);
        ret[propKey] = qba;
        // TODO convert known data types

        XFree(data);
    }

    return ret;
}

Window WindowProperties::GetWindow()
{
    return this->w;
}

QString WindowProperties::LookupAtom(Atom a)
{
    static QMap<Atom, QString> cache;

    if (! cache.contains(a)) {
        char * atomName = XGetAtomName(QX11Info::display(), a);
        if (atomName == nullptr) {
            return QString();
        }

        cache[a] = QString(atomName);
        XFree(atomName);
    }

    return cache[a];
}
